@@ -1,7 +1,6 @@
 import numpy as np
 
 def read_encoded_csv(csv_path):
-
     print('[LOG] Reading file: ' + csv_path)
     try:
         data_file = open(csv_path, "r")
@@ -15,7 +14,16 @@ def read_encoded_csv(csv_path):
             for x in row:
                 float_row.append(float(x))
             data.append(float_row)
+        data_file.close()
         return np.array(features), np.array(data)
     except:
         print(f'[ERROR] Failed to read file {csv_path}')
-        return None, None
\ No newline at end of file
+        return None, None
+    
+def remove_non_mutual_features(features_R, features_S, data_R, data_S):
+    matching_features = set(features_R).intersection(features_S)
+    
+    idx_R = [i for i, feature in enumerate(features_R) if feature in matching_features]
+    idx_S = [i for i, feature in enumerate(features_S) if feature in matching_features]
+    
+    return np.array(list(matching_features)), data_R[:,idx_R], data_S[:,idx_S]
\ No newline at end of file
